@@ -131,6 +131,7 @@ static lcp_signature_t *read_pubkey_file(const char *file)
     }
 
     RSA *pubkey = PEM_read_RSA_PUBKEY(fp, NULL, NULL, NULL);
+    fclose(fp);
     if ( pubkey == NULL ) {
         ERR_load_crypto_strings();
         ERROR("Error: failed to read .pem file %s: %s\n", file,
@@ -155,14 +156,18 @@ static lcp_signature_t *read_pubkey_file(const char *file)
 
     memset(sig, 0, sizeof(*sig) + 2*keysize);
     sig->pubkey_size = keysize;
-    if ( (unsigned int)BN_num_bytes(pubkey->n) != keysize ) {
-        ERROR("Error: modulus size not match key size\n");
-        free(sig);
-        RSA_free(pubkey);
-        return NULL;
-    }
+
+    const BIGNUM *modulus = NULL;
+
+    /* OpenSSL Version 1.1.0 and later don't allow direct access to RSA 
+       stuct */ 
+    #if OPENSSL_VERSION_NUMBER >= 0x10100000L
+        RSA_get0_key(pubkey, &modulus, NULL, NULL); 
+    #else
+        modulus = pubkey->n;
+    #endif
     unsigned char key[keysize];
-    BN_bn2bin(pubkey->n, key);
+    BN_bn2bin(modulus, key);
     /* openssl key is big-endian and policy requires little-endian, so reverse
        bytes */
     for ( unsigned int i = 0; i < keysize; i++ )
@@ -172,7 +177,7 @@ static lcp_signature_t *read_pubkey_file(const char *file)
         LOG("signature:\n");
         display_signature("    ", sig, false);
     }
-
+ 
     RSA_free(pubkey);
     return sig;
 }
@@ -195,6 +200,7 @@ static bool sign_list_data(lcp_policy_list_t *pollist, const char *privkey_file)
     }
 
     RSA *privkey = PEM_read_RSAPrivateKey(fp, NULL, NULL, NULL);
+    fclose(fp);
     if ( privkey == NULL ) {
         ERR_load_crypto_strings();
         ERROR("Error: failed to read .pem file %s: %s\n", privkey_file,
@@ -274,11 +280,15 @@ static int create(void)
         }
         if ( !verify_policy_element(elt, len) ) {
             free(pollist);
+            free(elt);
             return 1;
         }
         pollist = add_policy_element(pollist, elt);
-        if ( pollist == NULL )
+        if ( pollist == NULL ) {
+            free(elt);
             return 1;
+        }
+        free(elt);
     }
 
     bool write_ok = write_policy_list_file(pollist_file, pollist);
@@ -320,8 +330,16 @@ static int sign(void)
     }
     pollist->sig_alg = LCP_POLSALG_RSA_PKCS_15;
 
-    if ( no_sigblock )
-        memset(get_sig_block(pollist), 0, sig->pubkey_size);
+    if ( no_sigblock ) {
+        unsigned char *sigblock = get_sig_block(pollist);
+        if ( sigblock == NULL ) {
+            ERROR("Error: failed to get signature block\n");
+            free(sig);
+            free(pollist);
+            return 1;
+        }
+        memset(sigblock, 0, sig->pubkey_size);
+    }
     else {
         if ( !sign_list_data(pollist, privkey_file) ) {
             free(sig);
