@@ -242,7 +242,7 @@ const char *hash_alg_to_str(uint8_t alg)
     static const char *alg_str[] = { "LCP_POLHALG_SHA1" };
     static char buf[32];
 
-    if ( alg > ARRAY_SIZE(alg_str) ) {
+    if ( alg >= ARRAY_SIZE(alg_str) ) {
         snprintf(buf, sizeof(buf), "unknown (%u)", alg);
         return buf;
     }
@@ -274,14 +274,24 @@ bool verify_signature(const uint8_t *data, size_t data_size,
         ERROR("Error: failed to allocate key\n");
         return false;
     }
-    rsa_pubkey->n = BN_bin2bn(key, pubkey_size, NULL);
+    
+    BIGNUM *modulus = BN_bin2bn(key, pubkey_size, NULL);
+    BIGNUM *exponent = BN_new();
 
     /* uses fixed exponent (LCP_SIG_EXPONENT) */
     char exp[32];
     snprintf(exp, sizeof(exp), "%u", LCP_SIG_EXPONENT);
-    rsa_pubkey->e = NULL;
-    BN_dec2bn(&rsa_pubkey->e, exp);
-    rsa_pubkey->d = rsa_pubkey->p = rsa_pubkey->q = NULL;
+    BN_dec2bn(&exponent, exp);
+    
+    /* OpenSSL Version 1.1.0 and later don't allow direct access to RSA 
+       stuct */ 
+    #if OPENSSL_VERSION_NUMBER >= 0x10100000L
+        RSA_set0_key(rsa_pubkey, modulus, exponent, NULL); 
+    #else
+      	rsa_pubkey->n = modulus;
+    	rsa_pubkey->e = exponent;
+  	rsa_pubkey->d = rsa_pubkey->p = rsa_pubkey->q = NULL;
+    #endif
 
     /* first create digest of data */
     tb_hash_t digest;
@@ -327,10 +337,10 @@ bool verify_signature(const uint8_t *data, size_t data_size,
         ERROR("Error: failed to verify list: %s\n", 
               ERR_error_string(ERR_get_error(), NULL));
         ERR_free_strings();
-        RSA_free(rsa_pubkey);
+	RSA_free(rsa_pubkey);
         return false;
     }
-
+    
     RSA_free(rsa_pubkey);
     return true;
 }
