@@ -32,7 +32,7 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  */
-
+#ifndef IS_INCLUDED
 #include <config.h>
 #include <types.h>
 #include <stdbool.h>
@@ -49,7 +49,7 @@
 #include <multiboot.h>
 #include <txt/config_regs.h>
 #include <txt/heap.h>
-
+#endif
 /*
  * copy of original command line
  * part of tboot measurement (hence in .text section)
@@ -83,7 +83,9 @@ static const cmdline_option_t g_tboot_cmdline_options[] = {
     { "min_ram", "0" },              /* size in bytes | 0 for no min */
     { "call_racm", "false" },        /* true|false|check */
     { "measure_nv", "false" },       /* true|false */
-    { "extpol",    "sha1" },        /* agile|embedded|sha1|sha256|sm3|... */
+    { "extpol",    "sha1" },         /*agile|embedded|sha1|sha256|sm3|... */
+    { "ignore_prev_err", "true"},    /* true|false */
+    { "force_tpm2_legacy_log", "false"}, /* true|false */
     { NULL, NULL }
 };
 static char g_tboot_param_values[ARRAY_SIZE(g_tboot_cmdline_options)][MAX_VALUE_LEN];
@@ -499,35 +501,57 @@ bool get_tboot_measure_nv(void)
     return true;
 }
 
-
 void get_tboot_extpol(void)
 {
-    const char *extpol = get_option_val(g_tboot_cmdline_options,  g_tboot_param_values, "extpol");
+    const char *extpol = get_option_val(g_tboot_cmdline_options, g_tboot_param_values, "extpol");
+    struct tpm_if *tpm = get_tpm();
 
     if ( extpol == NULL ) {
-        g_tpm->extpol = TB_EXTPOL_FIXED;
-        g_tpm->cur_alg = TB_HALG_SHA256;
+        tpm->extpol = TB_EXTPOL_FIXED;
+        tpm->cur_alg = TB_HALG_SHA256;
         return;
     }
 
     if ( strcmp(extpol, "agile") == 0 ) {
-        g_tpm->extpol = TB_EXTPOL_AGILE;
-        g_tpm->cur_alg = TB_HALG_SHA256;
+        tpm->extpol = TB_EXTPOL_AGILE;
+        tpm->cur_alg = TB_HALG_SHA256;
     } else if ( strcmp(extpol, "embedded") == 0 ) {
-        g_tpm->extpol = TB_EXTPOL_EMBEDDED;
-        g_tpm->cur_alg = TB_HALG_SHA256;
+        tpm->extpol = TB_EXTPOL_EMBEDDED;
+        tpm->cur_alg = TB_HALG_SHA256;
     } else if ( strcmp(extpol, "sha256") == 0 ) {
-        g_tpm->extpol = TB_EXTPOL_FIXED;
-        g_tpm->cur_alg = TB_HALG_SHA256;
+        tpm->extpol = TB_EXTPOL_FIXED;
+        tpm->cur_alg = TB_HALG_SHA256;
     } else if ( strcmp(extpol, "sha1") == 0 ) {
-        g_tpm->extpol = TB_EXTPOL_FIXED;
-        g_tpm->cur_alg = TB_HALG_SHA1;
+        tpm->extpol = TB_EXTPOL_FIXED;
+        tpm->cur_alg = TB_HALG_SHA1;
     } else if ( strcmp(extpol, "sm3") == 0 ) {
-        g_tpm->extpol = TB_EXTPOL_FIXED;
-        g_tpm->cur_alg = TB_HALG_SM3;
+        tpm->extpol = TB_EXTPOL_FIXED;
+        tpm->cur_alg = TB_HALG_SM3;
     }
 }
 
+bool get_tboot_force_tpm2_legacy_log(void)
+{
+    const char *force_legacy_log =
+       get_option_val(g_tboot_cmdline_options,
+              g_tboot_param_values,
+              "force_tpm2_legacy_log");
+    if ( force_legacy_log != NULL && strcmp(force_legacy_log, "true") == 0 )
+       return true;
+    return false;
+}
+
+bool get_tboot_ignore_prev_err(void)
+{
+    const char *ignore_prev_err = 
+	    get_option_val(g_tboot_cmdline_options,
+			   g_tboot_param_values,
+			   "ignore_prev_err");
+    if ( ignore_prev_err == NULL || strcmp(ignore_prev_err, "true") == 0 )
+	    return true;
+    return false;
+}
+
 /*
  * linux kernel command line parsing
  */
