@@ -42,12 +42,12 @@
 #include <compiler.h>
 #include <string.h>
 #include <hash.h>
-#include <tpm.h>
 #include <tboot.h>
 #include <tb_policy.h>
 #include <tb_error.h>
 #include <vmac.h>
 #include <integrity.h>
+#include <tpm.h>
 
 #include <page.h>
 #include <paging.h>
@@ -60,24 +60,24 @@ __data pre_k_s3_state_t g_pre_k_s3_state;
 __data post_k_s3_state_t g_post_k_s3_state;
 
 /* state sealed before extending PCRs and launching kernel */
-static __data uint8_t  sealed_pre_k_state[512];
+static __data uint8_t  sealed_pre_k_state[2048];
 static __data uint32_t sealed_pre_k_state_size;
 
 /* state sealed just before entering S3 (after kernel shuts down) */
-static __data uint8_t  sealed_post_k_state[512];
+static __data uint8_t  sealed_post_k_state[2048];
 static __data uint32_t sealed_post_k_state_size;
 
 /* PCR 17+18 values post-launch and before extending (used to seal verified
    launch hashes and memory integrity UMAC) */
-static __data tpm_pcr_value_t post_launch_pcr17, post_launch_pcr18;
+__data tpm_pcr_value_t post_launch_pcr17, post_launch_pcr18;
 
 extern tboot_shared_t _tboot_shared;
 
-extern bool hash_policy(tb_hash_t *hash, uint8_t hash_alg);
+extern bool hash_policy(tb_hash_t *hash, uint16_t hash_alg);
 extern void apply_policy(tb_error_t error);
 
 #define EVTTYPE_TB_MEASUREMENT (0x400 + 0x101)
-extern bool evtlog_append(uint8_t pcr, tb_hash_t *hash, uint32_t type);
+extern bool evtlog_append(uint8_t pcr, hash_list_t *hl, uint32_t type);
 
 typedef struct {
     uint8_t mac_key[VMAC_KEY_LEN/8];
@@ -87,66 +87,57 @@ typedef struct {
 
 static bool extend_pcrs(void)
 {
-    tpm_pcr_value_t pcr17, pcr18;
-
-    tpm_pcr_read(2, 17, &pcr17);
-    tpm_pcr_read(2, 18, &pcr18);
-    printk("PCRs before extending:\n");
-    printk("  PCR 17: "); print_hash((tb_hash_t *)&pcr17, TB_HALG_SHA1);
-    printk("  PCR 18: "); print_hash((tb_hash_t *)&pcr18, TB_HALG_SHA1);
-
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
     for ( int i = 0; i < g_pre_k_s3_state.num_vl_entries; i++ ) {
-        if ( tpm_pcr_extend(2, g_pre_k_s3_state.vl_entries[i].pcr,
-                     (tpm_pcr_value_t *)&g_pre_k_s3_state.vl_entries[i].hash,
-                     NULL) != TPM_SUCCESS )
+        if ( !tpm_fp->pcr_extend(tpm, 2, g_pre_k_s3_state.vl_entries[i].pcr,
+                    &g_pre_k_s3_state.vl_entries[i].hl) )
             return false;
         if ( !evtlog_append(g_pre_k_s3_state.vl_entries[i].pcr,
-                            &g_pre_k_s3_state.vl_entries[i].hash,
+                            &g_pre_k_s3_state.vl_entries[i].hl,
                             EVTTYPE_TB_MEASUREMENT) )
             return false;
     }
 
-    tpm_pcr_read(2, 17, &pcr17);
-    tpm_pcr_read(2, 18, &pcr18);
-    printk("PCRs after extending:\n");
-    printk("  PCR 17: "); print_hash((tb_hash_t *)&pcr17, TB_HALG_SHA1);
-    printk("  PCR 18: "); print_hash((tb_hash_t *)&pcr18, TB_HALG_SHA1);
-
     return true;
 }
 
 static void print_pre_k_s3_state(void)
 {
-    printk("pre_k_s3_state:\n");
-    printk("\t vtd_pmr_lo_base: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_lo_base);
-    printk("\t vtd_pmr_lo_size: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_lo_size);
-    printk("\t vtd_pmr_hi_base: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_hi_base);
-    printk("\t vtd_pmr_hi_size: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_hi_size);
-    printk("\t pol_hash: ");
-    print_hash(&g_pre_k_s3_state.pol_hash, TB_HALG_SHA1);
-    printk("\t VL measurements:\n");
-    for ( int i = 0; i < g_pre_k_s3_state.num_vl_entries; i++ ) {
-        printk("\t   PCR %d: ", g_pre_k_s3_state.vl_entries[i].pcr);
-        print_hash(&g_pre_k_s3_state.vl_entries[i].hash, TB_HALG_SHA1);
+    struct tpm_if *tpm = get_tpm();
+    
+    printk(TBOOT_DETA"pre_k_s3_state:\n");
+    printk(TBOOT_DETA"\t vtd_pmr_lo_base: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_lo_base);
+    printk(TBOOT_DETA"\t vtd_pmr_lo_size: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_lo_size);
+    printk(TBOOT_DETA"\t vtd_pmr_hi_base: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_hi_base);
+    printk(TBOOT_DETA"\t vtd_pmr_hi_size: 0x%Lx\n", g_pre_k_s3_state.vtd_pmr_hi_size);
+    printk(TBOOT_DETA"\t pol_hash: ");
+    print_hash(&g_pre_k_s3_state.pol_hash, tpm->cur_alg);
+    printk(TBOOT_DETA"\t VL measurements:\n");
+    for ( unsigned int i = 0; i < g_pre_k_s3_state.num_vl_entries; i++ ) {
+        printk(TBOOT_DETA"\t   PCR %d (alg count %d):\n",
+                g_pre_k_s3_state.vl_entries[i].pcr,
+                g_pre_k_s3_state.vl_entries[i].hl.count);
+        for ( unsigned int j = 0; j < g_pre_k_s3_state.vl_entries[i].hl.count; j++ ) {
+            printk(TBOOT_DETA"\t\t   alg %04X: ",
+                    g_pre_k_s3_state.vl_entries[i].hl.entries[j].alg);
+            print_hash(&g_pre_k_s3_state.vl_entries[i].hl.entries[j].hash,
+                    g_pre_k_s3_state.vl_entries[i].hl.entries[j].alg);
+        }
     }
 }
 
 static void print_post_k_s3_state(void)
 {
-    printk("post_k_s3_state:\n");
-    printk("\t kernel_s3_resume_vector: 0x%Lx\n",
+    printk(TBOOT_DETA"post_k_s3_state:\n");
+    printk(TBOOT_DETA"\t kernel_s3_resume_vector: 0x%Lx\n",
            g_post_k_s3_state.kernel_s3_resume_vector);
-    printk("\t kernel_integ: ");
+    printk(TBOOT_DETA"\t kernel_integ: ");
     print_hex(NULL, &g_post_k_s3_state.kernel_integ,
               sizeof(g_post_k_s3_state.kernel_integ));
 }
 
-static bool seal_data(const void *data, size_t data_size,
-                   const void *secrets, size_t secrets_size,
-                   uint8_t *sealed_data, uint32_t *sealed_data_size,
-                   const uint8_t pcr_indcs_create[], int nr_pcr_indcs_create,
-                   const uint8_t pcr_indcs_release[], int nr_pcr_indcs_release,
-                   const tpm_pcr_value_t *pcr_values_release[])
+static bool seal_data(const void *data, size_t data_size, const void *secrets, size_t secrets_size, uint8_t *sealed_data, uint32_t *sealed_data_size)
 {
     /* TPM_Seal can only seal small data (like key or hash), so hash data */
     struct __packed {
@@ -154,35 +145,27 @@ static bool seal_data(const void *data, size_t data_size,
         uint8_t   secrets[secrets_size];
     } blob;
     uint32_t err;
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
 
     memset(&blob, 0, sizeof(blob));
-    if ( !hash_buffer(data, data_size, &blob.data_hash, TB_HALG_SHA1) ) {
-        printk("failed to hash data\n");
+    if ( !hash_buffer(data, data_size, &blob.data_hash, tpm->cur_alg) ) {
+        printk(TBOOT_ERR"failed to hash data\n");
         return false;
     }
 
-    if ( secrets != NULL && secrets_size > 0 )
-        memcpy(blob.secrets, secrets, secrets_size);
+    if ( secrets != NULL && secrets_size > 0 )  memcpy(blob.secrets, secrets, secrets_size);
 
-    err = tpm_seal(2, TPM_LOC_TWO,
-                   nr_pcr_indcs_create, pcr_indcs_create,
-                   nr_pcr_indcs_release, pcr_indcs_release,
-                   pcr_values_release,
-                   sizeof(blob), (const uint8_t *)&blob,
-                   sealed_data_size, sealed_data);
-    if ( err != TPM_SUCCESS )
-        printk("failed to seal data\n");
+    err = tpm_fp->seal(tpm, 2, sizeof(blob), (const uint8_t *)&blob, sealed_data_size, sealed_data);
+    if ( !err )  printk(TBOOT_WARN"failed to seal data\n");
 
     /* since blob might contain secret, clear it */
     memset(&blob, 0, sizeof(blob));
 
-    return (err == TPM_SUCCESS) ? true : false;
+    return err;
 }
 
-static bool verify_sealed_data(const uint8_t *sealed_data,
-                               uint32_t sealed_data_size,
-                               const void *curr_data, size_t curr_data_size,
-                               void *secrets, size_t secrets_size)
+static bool verify_sealed_data(const uint8_t *sealed_data,  uint32_t sealed_data_size, const void *curr_data, size_t curr_data_size, void *secrets, size_t secrets_size)
 {
     /* sealed data is hash of state data and optional secret */
     struct __packed {
@@ -190,28 +173,28 @@ static bool verify_sealed_data(const uint8_t *sealed_data,
         uint8_t   secrets[secrets_size];
     } blob;
     bool err = true;
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
 
     uint32_t data_size = sizeof(blob);
-    if ( tpm_unseal(2, sealed_data_size, sealed_data,
-                    &data_size, (uint8_t *)&blob) != TPM_SUCCESS ) {
-        printk("failed to unseal blob\n");
+    if ( !tpm_fp->unseal(tpm, tpm->cur_loc, sealed_data_size, sealed_data, &data_size, (uint8_t *)&blob) ) {
+        printk(TBOOT_ERR"failed to unseal blob\n");
         return false;
     }
     if ( data_size != sizeof(blob) ) {
-        printk("unsealed state data size mismatch\n");
+        printk(TBOOT_WARN"unsealed state data size mismatch\n");
         goto done;
     }
 
     /* verify that (hash of) current data maches sealed hash */
     tb_hash_t curr_data_hash;
     memset(&curr_data_hash, 0, sizeof(curr_data_hash));
-    if ( !hash_buffer(curr_data, curr_data_size, &curr_data_hash,
-                      TB_HALG_SHA1) ) {
-        printk("failed to hash state data\n");
+    if ( !hash_buffer(curr_data, curr_data_size, &curr_data_hash, tpm->cur_alg) ) {
+        printk(TBOOT_WARN"failed to hash state data\n");
         goto done;
     }
-    if ( !are_hashes_equal(&blob.data_hash, &curr_data_hash, TB_HALG_SHA1) ) {
-        printk("sealed hash does not match current hash\n");
+    if ( !are_hashes_equal(&blob.data_hash, &curr_data_hash, tpm->cur_alg) ) {
+        printk(TBOOT_WARN"sealed hash does not match current hash\n");
         goto done;
     }
 
@@ -233,32 +216,29 @@ static bool verify_sealed_data(const uint8_t *sealed_data,
  */
 bool seal_pre_k_state(void)
 {
-    const uint8_t pcr_indcs_create[]  = {17, 18};
-    const uint8_t pcr_indcs_release[] = {17, 18};
-    const tpm_pcr_value_t *pcr_values_release[] = {&post_launch_pcr17,
-                                                   &post_launch_pcr18};
-
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
+    
     /* save hash of current policy into g_pre_k_s3_state */
     memset(&g_pre_k_s3_state.pol_hash, 0, sizeof(g_pre_k_s3_state.pol_hash));
-    if ( !hash_policy(&g_pre_k_s3_state.pol_hash, TB_HALG_SHA1) ) {
-        printk("failed to hash policy\n");
+    if ( !hash_policy(&g_pre_k_s3_state.pol_hash, tpm->cur_alg) ) {
+        printk(TBOOT_ERR"failed to hash policy\n");
         goto error;
     }
 
     print_pre_k_s3_state();
 
-    /* read PCR 17/18 */
-    if ( tpm_pcr_read(2, 17, &post_launch_pcr17) != TPM_SUCCESS ||
-         tpm_pcr_read(2, 18, &post_launch_pcr18) != TPM_SUCCESS )
-        goto error;
+    /* read PCR 17/18, only for tpm1.2 */
+    if ( tpm->major == TPM12_VER_MAJOR ) {
+        if ( !tpm_fp->pcr_read(tpm, 2, 17, &post_launch_pcr17) ||
+             !tpm_fp->pcr_read(tpm, 2, 18, &post_launch_pcr18) )
+            goto error;
+    }
 
     sealed_pre_k_state_size = sizeof(sealed_pre_k_state);
     if ( !seal_data(&g_pre_k_s3_state, sizeof(g_pre_k_s3_state),
                     NULL, 0,
-                    sealed_pre_k_state, &sealed_pre_k_state_size,
-                    pcr_indcs_create, ARRAY_SIZE(pcr_indcs_create),
-                    pcr_indcs_release, ARRAY_SIZE(pcr_indcs_release),
-                    pcr_values_release) )
+                    sealed_pre_k_state, &sealed_pre_k_state_size) )
         goto error;
 
     /* we can't leave the system in a state without valid measurements of
@@ -300,7 +280,7 @@ static bool measure_memory_integrity(vmac_t *mac, uint8_t key[VMAC_KEY_LEN/8])
 
         /* overflow? */
         if ( plus_overflow_u64(start, _tboot_shared.mac_regions[i].size) ) {
-            printk("start plus size overflows during MACing\n");
+            printk(TBOOT_ERR"start plus size overflows during MACing\n");
             return false;
         }
 
@@ -312,14 +292,14 @@ static bool measure_memory_integrity(vmac_t *mac, uint8_t key[VMAC_KEY_LEN/8])
 
         /* overflow? */
         if ( plus_overflow_u64(end, 1) ) {
-            printk("end up to the alignment overflows during MACing\n");
+            printk(TBOOT_ERR"end up to the alignment overflows during MACing\n");
             return false;
         }
 
         /* if not overflow, we get end aligned */
         end++;
 
-        printk("MACing region %u:  0x%Lx - 0x%Lx\n", i, start, end);
+        printk(TBOOT_DETA"MACing region %u:  0x%Lx - 0x%Lx\n", i, start, end);
 
         /*
          * spfn: start pfn in 2-Mbyte page
@@ -328,7 +308,7 @@ static bool measure_memory_integrity(vmac_t *mac, uint8_t key[VMAC_KEY_LEN/8])
          */
         /* check overflow? */
         if ( plus_overflow_u64(end, MAC_PAGE_SIZE) ) {
-            printk("end plus MAC_PAGE_SIZE overflows during MACing\n");
+            printk(TBOOT_ERR"end plus MAC_PAGE_SIZE overflows during MACing\n");
             return false;
         }
 
@@ -401,47 +381,36 @@ static bool measure_memory_integrity(vmac_t *mac, uint8_t key[VMAC_KEY_LEN/8])
  */
 bool verify_integrity(void)
 {
-    unsigned int i;
-    uint8_t pcr_indcs_create[] = {17, 18};
-    tpm_pcr_value_t  pcr17, pcr18;
-    const tpm_pcr_value_t *pcr_values_create[] = {&pcr17, &pcr18};
-    tpm_pcr_value_t cap_val;   /* use whatever val is on stack */
+    tpm_pcr_value_t pcr17, pcr18;
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
 
-    tpm_pcr_read(2, 17, &pcr17);
-    tpm_pcr_read(2, 18, &pcr18);
-    printk("PCRs before unseal:\n");
-    printk("  PCR 17: "); print_hash((tb_hash_t *)&pcr17, TB_HALG_SHA1);
-    printk("  PCR 18: "); print_hash((tb_hash_t *)&pcr18, TB_HALG_SHA1);
+    /* read PCR 17/18, only for tpm1.2 */
+    if ( tpm->major == TPM12_VER_MAJOR ) {
+        if ( !tpm_fp->pcr_read(tpm, 2, 17, &pcr17) ||
+             !tpm_fp->pcr_read(tpm, 2, 18, &pcr18) )
+            goto error;
+        printk(TBOOT_DETA"PCRs before unseal:\n");
+        printk(TBOOT_DETA"  PCR 17: ");
+        print_hash((tb_hash_t *)&pcr17, TB_HALG_SHA1);
+        printk(TBOOT_DETA"  PCR 18: ");
+        print_hash((tb_hash_t *)&pcr18, TB_HALG_SHA1);
+    }
 
     /* verify integrity of pre-kernel state data */
-    printk("verifying pre_k_s3_state\n");
+    printk(TBOOT_INFO"verifying pre_k_s3_state\n");
     if ( !verify_sealed_data(sealed_pre_k_state, sealed_pre_k_state_size,
                              &g_pre_k_s3_state, sizeof(g_pre_k_s3_state),
                              NULL, 0) )
         goto error;
 
-    /* to prevent rollback attack using old sealed measurements,
-       verify that (creation) PCRs at mem integrity seal time are same as
-       if we extend current PCRs with unsealed VL measurements */
-    /* TBD: we should check all DRTM PCRs */
-    for ( i = 0; i < g_pre_k_s3_state.num_vl_entries; i++ ) {
-        if ( g_pre_k_s3_state.vl_entries[i].pcr == 17 )
-            extend_hash((tb_hash_t *)&pcr17,
-                        &g_pre_k_s3_state.vl_entries[i].hash, TB_HALG_SHA1);
-        else if ( g_pre_k_s3_state.vl_entries[i].pcr == 18 )
-            extend_hash((tb_hash_t *)&pcr18,
-                        &g_pre_k_s3_state.vl_entries[i].hash, TB_HALG_SHA1);
-    }
-    if ( !tpm_cmp_creation_pcrs(ARRAY_SIZE(pcr_indcs_create),
-                                pcr_indcs_create, pcr_values_create,
-                                sealed_post_k_state_size,
-                                sealed_post_k_state) ) {
-        printk("extended PCR values don't match creation values in sealed blob.\n");
+    if ( !tpm_fp->verify_creation(tpm, sealed_post_k_state_size,  sealed_post_k_state) ) {
+        printk(TBOOT_ERR"extended PCR values don't match creation values in sealed blob.\n");
         goto error;
     }
 
     /* verify integrity of post-kernel state data */
-    printk("verifying post_k_s3_state\n");
+    printk(TBOOT_INFO"verifying post_k_s3_state\n");
     sealed_secrets_t secrets;
     if ( !verify_sealed_data(sealed_post_k_state, sealed_post_k_state_size,
                              &g_post_k_s3_state, sizeof(g_post_k_s3_state),
@@ -453,15 +422,15 @@ bool verify_integrity(void)
     if ( !measure_memory_integrity(&mac, secrets.mac_key) )
         goto error;
     if ( memcmp(&mac, &g_post_k_s3_state.kernel_integ, sizeof(mac)) ) {
-        printk("memory integrity lost on S3 resume\n");
-        printk("MAC of current image is: ");
+        printk(TBOOT_INFO"memory integrity lost on S3 resume\n");
+        printk(TBOOT_DETA"MAC of current image is: ");
         print_hex(NULL, &mac, sizeof(mac));
-        printk("MAC of pre-S3 image is: ");
+        printk(TBOOT_DETA"MAC of pre-S3 image is: ");
         print_hex(NULL, &g_post_k_s3_state.kernel_integ,
                   sizeof(g_post_k_s3_state.kernel_integ));
         goto error;
     }
-    printk("memory integrity OK\n");
+    printk(TBOOT_INFO"memory integrity OK\n");
 
     /* re-extend PCRs with VL measurements
        we can't leave the system in a state without valid measurements of
@@ -484,7 +453,7 @@ bool verify_integrity(void)
     /* since we can't leave the system without any measurments representing the
        code-about-to-execute, and yet there is no integrity of that code,
        just cap PCR 18 */
-    if ( tpm_pcr_extend(2, 18, &cap_val, NULL) != TPM_SUCCESS )
+    if ( !tpm_fp->cap_pcrs(tpm, tpm->cur_loc, 18) )
         apply_policy(TB_ERR_FATAL);
     return false;
 }
@@ -495,29 +464,23 @@ bool verify_integrity(void)
  */
 bool seal_post_k_state(void)
 {
-    const uint8_t pcr_indcs_create[]  = {17, 18};
-    const uint8_t pcr_indcs_release[] = {17, 18};
-    const tpm_pcr_value_t *pcr_values_release[] = {&post_launch_pcr17,
-                                                   &post_launch_pcr18};
     sealed_secrets_t secrets;
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
 
     /* since tboot relies on the module it launches for resource protection,
        that module should have at least one region for itself, otherwise
        it will not be protected against S3 resume attacks */
     if ( _tboot_shared.num_mac_regions == 0 ) {
-        printk("no memory regions to MAC\n");
+        printk(TBOOT_ERR"no memory regions to MAC\n");
         return false;
     }
 
     /* calculate the memory integrity hash */
     uint32_t key_size = sizeof(secrets.mac_key);
     /* key must be random and secret even though auth not necessary */
-    if ( tpm_get_random(2, secrets.mac_key, &key_size) != TPM_SUCCESS ||
-         key_size != sizeof(secrets.mac_key) )
-        return false;
-    if ( !measure_memory_integrity(&g_post_k_s3_state.kernel_integ,
-                                   secrets.mac_key) )
-        return false;
+    if ( !tpm_fp->get_random(tpm, tpm->cur_loc, secrets.mac_key, &key_size) ||key_size != sizeof(secrets.mac_key) ) return false;
+    if ( !measure_memory_integrity(&g_post_k_s3_state.kernel_integ, secrets.mac_key) ) return false;
 
     /* copy s3_key into secrets to be sealed */
     memcpy(secrets.shared_key, _tboot_shared.s3_key, sizeof(secrets.shared_key));
@@ -525,13 +488,7 @@ bool seal_post_k_state(void)
     print_post_k_s3_state();
 
     sealed_post_k_state_size = sizeof(sealed_post_k_state);
-    if ( !seal_data(&g_post_k_s3_state, sizeof(g_post_k_s3_state),
-                    &secrets, sizeof(secrets),
-                    sealed_post_k_state, &sealed_post_k_state_size,
-                    pcr_indcs_create, ARRAY_SIZE(pcr_indcs_create),
-                    pcr_indcs_release, ARRAY_SIZE(pcr_indcs_release),
-                    pcr_values_release) )
-        return false;
+    if ( !seal_data(&g_post_k_s3_state, sizeof(g_post_k_s3_state), &secrets, sizeof(secrets), sealed_post_k_state, &sealed_post_k_state_size) ) return false;
 
     /* wipe secrets from memory */
     memset(&secrets, 0, sizeof(secrets));
