@@ -3,6 +3,7 @@
  *           binaries
  *
  * Copyright (c) 2006-2013, Intel Corporation
+ * Copyright (c) 2016 Real-Time Systems GmbH
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,6 +56,7 @@
 #include <mle.h>
 #include <txt/acmod.h>
 #include <cmdline.h>
+#include <tpm.h>
 
 /* copy of kernel/VMM command line so that can append 'tboot=0x1234' */
 static char *new_cmdline = (char *)TBOOT_KERNEL_CMDLINE_ADDR;
@@ -64,7 +66,7 @@ extern tboot_shared_t _tboot_shared;
 
 /* multiboot struct saved so that post_launch() can use it (in tboot.c) */
 extern loader_ctx *g_ldr_ctx;
-
+extern bool get_elf_image_range(const elf_header_t *elf, void **start, void **end);
 extern bool is_elf_image(const void *image, size_t size);
 extern bool expand_elf_image(const elf_header_t *elf, void **entry_point);
 extern bool expand_linux_image(const void *linux_image, size_t linux_size,
@@ -74,7 +76,7 @@ extern bool jump_elf_image(const void *entry_point, uint32_t magic);
 extern bool jump_linux_image(const void *entry_point);
 extern bool is_sinit_acmod(const void *acmod_base, uint32_t acmod_size, 
                            bool quiet);
-
+extern void apply_policy(tb_error_t error);
 extern uint32_t g_mb_orig_size;
 
 #define LOADER_CTX_BAD(xctx) \
@@ -414,6 +416,14 @@ static void *remove_module(loader_ctx *lctx, void *mod_start)
         if ( mod_start == NULL ) {
             char *cmdline = get_cmdline(lctx);
             char *mod_string = get_module_cmd(lctx, m);
+            if ( cmdline == NULL ) {
+                printk(TBOOT_ERR"could not find cmdline\n");
+                return NULL;
+            }
+            if ( mod_string == NULL ) {
+                printk(TBOOT_ERR"could not find module cmdline\n");
+                return NULL;
+            }
             if ((strlen(mod_string)) > (strlen(cmdline))){
                 if (strlen(mod_string) >= TBOOT_KERNEL_CMDLINE_SIZE){
                     printk(TBOOT_ERR"No room to copy MB2 cmdline [%d < %d]\n",
@@ -769,6 +779,166 @@ unsigned long get_mbi_mem_end_mb1(const multiboot_info_t *mbi)
     return PAGE_UP(end);
 }
 
+/*
+ * Move all mbi components/modules/mbi to end of memory
+ */
+static bool move_modules_to_high_memory(loader_ctx  *lctx)
+{
+    uint32_t memRequired;
+    uint64_t max_ram_base, max_ram_size, ld_ceiling;
+
+    uint32_t module_count, mod_i, mods_remaining;
+    module_t *m;
+
+    if (LOADER_CTX_BAD(lctx))
+        return false;
+
+    /*  Determine the size of memory required to pack all modules together */
+    module_count = get_module_count(lctx);
+    for( memRequired=0, mod_i=0; mod_i < module_count; mod_i++ ){
+        m = get_module(lctx, mod_i);
+        memRequired += PAGE_UP(m->mod_end - m->mod_start);
+    }
+
+    /*  NOTE: the e820 map has been modified already to reserve critical
+        memory regions (tboot memory, etc ...). get_highest_sized_ram
+        will return a range that excludes critical memory regions. */
+    get_highest_sized_ram(  memRequired, 0x100000000ULL,
+                            &max_ram_base, &max_ram_size);
+    if(!max_ram_base || !max_ram_size){
+        printk(TBOOT_INFO"ERROR No memory area found for image relocation!\n");
+        printk(TBOOT_INFO"required 0x%X\n", memRequired);
+        return false;
+    }
+    printk(TBOOT_INFO"highest suitable area @ 0x%llX (size 0x%llX)\n",
+            max_ram_base, max_ram_size);
+    ld_ceiling = PAGE_DOWN(max_ram_base + max_ram_size);
+
+    /*  Move modules below the load ceiling upto the ceiling:
+        Prevent module corruption:
+        - Move modules in order of highest to lowest.
+        - Only move modules completely below the load ceiling.
+     */
+    for( mods_remaining=module_count; mods_remaining > 0; mods_remaining--){
+        uint32_t    highest_mod_i = 0,
+                    highest_mod_base = 0,
+                    highest_mod_end = 0;
+        bool mod_found = false;
+        for( mod_i=0; mod_i<module_count; mod_i++){
+            m = get_module(lctx, mod_i);
+            if( (m->mod_start < ld_ceiling) ){
+                if( (m->mod_end > highest_mod_end) || !mod_found ){
+                    highest_mod_i   = mod_i;
+                    highest_mod_base= m->mod_start;
+                    highest_mod_end = m->mod_end;
+                    mod_found = true;
+                }
+            }
+        }
+
+        /* If no modules found, all modules are above ld_ceiling, We're done. */
+        if( !mod_found )
+            break;
+
+        m = get_module(lctx, highest_mod_i);
+        /* only move the highest module if explicitly below the ceiling */
+        if(highest_mod_end < ld_ceiling){
+            uint32_t size = highest_mod_end - highest_mod_base;
+            uint32_t highest_mod_newbase = PAGE_DOWN(ld_ceiling-size);
+            printk(TBOOT_INFO"moving module %u (%u B) from 0x%08X to 0x%08X\n",
+                    highest_mod_i, size, highest_mod_base, highest_mod_newbase);
+            memcpy((void *)highest_mod_newbase, (void *)highest_mod_base, size);
+            m->mod_start= highest_mod_newbase;
+            m->mod_end  = highest_mod_newbase+size;
+        }
+        /* lower the celing to the base address of the highest module */
+        ld_ceiling = PAGE_DOWN(m->mod_start);
+    }
+    return true;
+}
+
+/*
+ * Move any mbi components/modules/mbi that just above the kernel
+ */
+static bool move_modules_above_elf_kernel(  loader_ctx      *lctx,
+                                            elf_header_t    *kernel_image)
+{
+    void *elf_start, *elf_end;
+    uint32_t ld_floor, ld_ceiling;
+
+    uint32_t module_count, mod_i, mods_remaining;
+    module_t *m;
+
+    if (LOADER_CTX_BAD(lctx))
+        return false;
+
+    /* get end address of loaded elf image */
+    if ( !get_elf_image_range(kernel_image, &elf_start, &elf_end) ){
+        printk(TBOOT_INFO"ERROR: failed to get elf image range\n");
+        return false;
+    }
+    printk(TBOOT_INFO"ELF kernel top is at 0x%X\n", (uint32_t)elf_end);
+
+    /*  compute the lowest base address of all the modules: the ld ceiling */
+    module_count = get_module_count(lctx);
+    for( mod_i=0, ld_ceiling=0; mod_i < module_count; mod_i++ ){
+        m = get_module(lctx,mod_i);
+        ld_ceiling = (ld_ceiling < m->mod_start)? m->mod_start : ld_ceiling;
+    }
+
+    /*  set ld_floor to the highest of tboot end address or the ELF-image
+        end address, and then page align */
+    ld_floor = get_tboot_mem_end();
+    ld_floor = (ld_floor < (uint32_t)elf_end)? (uint32_t)elf_end : ld_floor;
+    ld_floor = PAGE_UP(ld_floor);
+
+    /*  Ensures all modules are above ld_floor: only move mods down, never up.
+        Failing this check is an indication ELF-loading may have corrupted one
+        of the modules. */
+    if( (uint32_t)ld_floor > ld_ceiling ){
+        printk( TBOOT_INFO"Load floor (0x%08X) > load ceiling (0x%08X)\n",
+                ld_floor, ld_ceiling);
+        return false;
+    }
+
+    /*  the i-th iteration of this loop will
+    move the i-th lowest module in memory to ld_floor
+    and raise ld_floor to the new end address of the i-th lowest module */
+    for( mods_remaining = module_count; mods_remaining > 0; mods_remaining--){
+        uint32_t    lowest_mod_i = 0,
+                    lowest_mod_base = 0,
+                    lowest_mod_end = 0;
+        bool mod_found = false;
+        /* find the lowest module above the floor */
+        for( mod_i=0; mod_i<module_count; mod_i++){
+            m = get_module(lctx, mod_i);
+            if(m->mod_start >= ld_floor){
+                if( (m->mod_start < lowest_mod_base) || !mod_found  ){
+                    lowest_mod_i   = mod_i;
+                    lowest_mod_base= m->mod_start;
+                    lowest_mod_end = m->mod_end;
+                    mod_found = true;
+                }
+            }
+        }
+        m = get_module(lctx, lowest_mod_i);
+
+        /* only move the lowest module if not already at the floor */
+        if(lowest_mod_base > ld_floor){
+            uint32_t size = lowest_mod_end - lowest_mod_base;
+            uint32_t lowest_mod_newbase = ld_floor; /* already page aligned */
+            printk(TBOOT_INFO"moving module %u (%u B) from 0x%08X to 0x%08X\n",
+                    lowest_mod_i, size, lowest_mod_base, lowest_mod_newbase);
+            memcpy((void *)lowest_mod_newbase, (void *)lowest_mod_base, size);
+            m->mod_start= lowest_mod_newbase;
+            m->mod_end  = lowest_mod_newbase+size;
+        }
+        /* raise the floor to the end address of the lowest module */
+        ld_floor = PAGE_UP(m->mod_end);
+    }
+    return true;
+}
+
 static void fixup_modules(loader_ctx *lctx, size_t offset)
 {
     unsigned int module_count = get_module_count(lctx);
@@ -1193,6 +1363,32 @@ bool launch_kernel(bool is_measured_launch)
 
     void *kernel_entry_point;
     uint32_t mb_type = MB_NONE;
+    struct tpm_if *tpm = get_tpm();
+
+    if (g_tpm_family != TPM_IF_20_CRB ) {
+        if (!release_locality(tpm->cur_loc))
+            printk(TBOOT_ERR"Release TPM FIFO locality %d failed \n", tpm->cur_loc);
+    }
+    else {
+        if (!tpm_relinquish_locality_crb(tpm->cur_loc))
+            printk(TBOOT_ERR"Relinquish TPM CRB locality %d failed \n", tpm->cur_loc);
+        if (!tpm_workaround_crb())
+            printk(TBOOT_ERR"CRB workaround failed \n");
+    }
+
+    /* if using memory logging, reserve log area */
+    if ( g_log_targets & TBOOT_LOG_TARGET_MEMORY ) {
+        uint64_t base = TBOOT_SERIAL_LOG_ADDR;
+        uint64_t size = TBOOT_SERIAL_LOG_SIZE;
+        printk(TBOOT_INFO"reserving tboot memory log (%Lx - %Lx) in e820 table\n", base, (base + size - 1));
+        if ( !e820_protect_region(base, size, E820_RESERVED) )
+            apply_policy(TB_ERR_FATAL);
+    }
+
+    /* replace map in loader context with copy */
+    replace_e820_map(g_ldr_ctx);
+    printk(TBOOT_DETA"adjusted e820 map:\n");
+    print_e820_map();
 
     if ( !verify_loader_context(g_ldr_ctx) )
         return false;
@@ -1244,6 +1440,11 @@ bool launch_kernel(bool is_measured_launch)
         
         /* fix for GRUB2, which may load modules into memory before tboot */
         move_modules(g_ldr_ctx);
+
+        /* move modules out of the way (to top og memory below 4G) */
+        printk(TBOOT_INFO"move modules to high memory\n");
+        if(!move_modules_to_high_memory(g_ldr_ctx))
+            return false;
     }
     else {
         printk(TBOOT_INFO"assuming kernel is Linux format\n");
@@ -1262,6 +1463,11 @@ bool launch_kernel(bool is_measured_launch)
         if ( !expand_elf_image((elf_header_t *)kernel_image,
                                &kernel_entry_point) )
             return false;
+
+        /* move modules on top of expanded kernel */
+        if(!move_modules_above_elf_kernel(g_ldr_ctx, (elf_header_t *)kernel_image))
+            return false;
+
         printk(TBOOT_INFO"transfering control to kernel @%p...\n", 
                kernel_entry_point);
         /* (optionally) pause when transferring to kernel */
@@ -1272,9 +1478,18 @@ bool launch_kernel(bool is_measured_launch)
                               MB_MAGIC : MB2_LOADER_MAGIC);
     }
     else if ( kernel_type == LINUX ) {
-        m = get_module(g_ldr_ctx,0);
-        void *initrd_image = (void *)m->mod_start;
-        size_t initrd_size = m->mod_end - m->mod_start;
+        void *initrd_image;
+        size_t initrd_size;
+
+        if ( get_module_count(g_ldr_ctx) == 0 ) {
+            initrd_size = 0;
+            initrd_image = 0;
+        }
+        else {
+            m = get_module(g_ldr_ctx,0);
+            initrd_image = (void *)m->mod_start;
+            initrd_size = m->mod_end - m->mod_start;
+        }
 
         expand_linux_image(kernel_image, kernel_size,
                            initrd_image, initrd_size,
@@ -1660,6 +1875,10 @@ replace_e820_map(loader_ctx *lctx)
              */
             new = get_e820_copy();
             old = get_loader_memmap(lctx);
+            if ( old == NULL ) {
+                printk(TBOOT_ERR"old memory map not found\n");
+                return;
+            }
             for (i = 0; i < (get_nr_map()); i++){
                 *old = *new;
                 old++, new++;
@@ -1768,6 +1987,30 @@ get_loader_efi_ptr(loader_ctx *lctx, uint32_t *address, uint64_t *long_address)
     return false;
 }
 
+
+uint32_t
+find_efi_memmap(loader_ctx *lctx, uint32_t *descr_size,
+                uint32_t *descr_vers, uint32_t *mmap_size) {
+    struct mb2_tag *start = NULL, *hit = NULL;
+    struct mb2_tag_efi_mmap *efi_mmap = NULL;
+
+    start = (struct mb2_tag *)(lctx->addr + 8);
+    hit = find_mb2_tag_type(start, MB2_TAG_TYPE_EFI_MMAP);
+    if (hit == NULL) {
+       return 0;
+    }
+
+    efi_mmap = (struct mb2_tag_efi_mmap *)hit;
+    *descr_size = efi_mmap->descr_size;
+    *descr_vers = efi_mmap->descr_vers;
+    *mmap_size = efi_mmap->size - sizeof(struct mb2_tag_efi_mmap);
+    if (*mmap_size % *descr_size) {
+        printk(TBOOT_WARN "EFI memmmap (0x%x) should be a multiple of descriptor size (0x%x)\n",
+	       *mmap_size, *descr_size);
+    }
+    return (uint32_t)(&efi_mmap->efi_mmap);
+}
+
 bool
 is_loader_launch_efi(loader_ctx *lctx)
 {
