@@ -59,13 +59,16 @@ __data tboot_log_t *g_log = NULL;
 
 static void memlog_init(void)
 {
-        if ( g_log == NULL ) {
-        g_log = (tboot_log_t *)TBOOT_SERIAL_LOG_ADDR;
-        g_log->uuid = (uuid_t)TBOOT_LOG_UUID;
-        g_log->curr_pos = 0;
-        g_log->zip_pos = 0;
-        g_log->zip_size = 0;
-        }
+
+
+   if ( g_log == NULL ) {
+       g_log = (tboot_log_t *)TBOOT_SERIAL_LOG_ADDR;
+       g_log->uuid = (uuid_t)TBOOT_LOG_UUID;
+       g_log->curr_pos = 0;
+       g_log->zip_count = 0;
+       for ( uint8_t i = 0; i < ZIP_COUNT_MAX; i++ ) g_log->zip_pos[i] = 0;
+       for ( uint8_t i = 0; i < ZIP_COUNT_MAX; i++ ) g_log->zip_size[i] = 0;
+       }
 
     /* initialize these post-launch as well, since bad/malicious values */
     /* could compromise environment */
@@ -73,37 +76,82 @@ static void memlog_init(void)
     g_log->max_size = TBOOT_SERIAL_LOG_SIZE - sizeof(*g_log);
 
     /* if we're calling this post-launch, verify that curr_pos is valid */
-    if ( g_log->zip_pos > g_log->max_size ){
-        g_log->zip_pos = 0;
+    if ( g_log->zip_pos[g_log->zip_count] > g_log->max_size ){
         g_log->curr_pos = 0;
+        g_log->zip_count = 0;
+        for ( uint8_t i = 0; i < ZIP_COUNT_MAX; i++ ) g_log->zip_pos[i] = 0;
+        for ( uint8_t i = 0; i < ZIP_COUNT_MAX; i++ ) g_log->zip_size[i] = 0;
     }
     if ( g_log->curr_pos > g_log->max_size )
-        g_log->curr_pos = g_log->zip_pos;
+        g_log->curr_pos = g_log->zip_pos[g_log->zip_count];
 }
 
 static void memlog_write( const char *str, unsigned int count)
 {
     /* allocate a 32K temp buffer for compressed log  */
-    char buf[32*1024];
+    static char buf[32*1024];
     char *out=buf;
+    int zip_size;
+    uint32_t zip_pos;
+    bool log_reset_flag;
 
     if ( g_log == NULL || count > g_log->max_size )
         return;
 
-    /* do a compression of the log if curr_pos + count > max_size  */
-    if (g_log->curr_pos + count > g_log->max_size) {
-       g_log->zip_size = LZ_Compress(&g_log->buf[g_log->zip_pos], out, g_log->curr_pos - g_log->zip_pos);
-       memcpy(&g_log->buf[g_log->zip_pos], out, g_log->zip_size); 
-       g_log->zip_pos += g_log->zip_size;
-       g_log->curr_pos = g_log->zip_pos;
-       if ( g_log->buf[g_log->zip_pos - 1] !='0')
-          g_log->buf[g_log->zip_pos] = '0';
-       else {
-          g_log->zip_pos--;
-          g_log->curr_pos--;
-       } 
-           
+    /* Check if there is space for the new string and a null terminator  */
+    if (g_log->curr_pos + count + 1> g_log->max_size) {
+
+        /* If there are space issues, only then log will be reset */
+        log_reset_flag = false;
+
+        /* Check if there is space to add another compressed chunk */
+        if(g_log->zip_count >= ZIP_COUNT_MAX)
+            log_reset_flag = true;
+        else{
+            /* Get the start position of the new compressed chunk */
+            zip_pos = g_log->zip_pos[g_log->zip_count];
+
+            /*  Compress the last part of the log buffer that is not compressed,
+                and put the compressed output in out (buf) */
+            zip_size = LZ_Compress(&g_log->buf[zip_pos], out, (g_log->curr_pos - zip_pos), sizeof(buf) );
+
+            /* Check if buf was large enough for LZ_compress to succeed */
+            if( zip_size < 0 )
+                log_reset_flag = true;
+            else{
+                /*  Check if there is space to add the compressed string, the
+                    new string and a null terminator to the log */
+                if( (zip_pos + zip_size + count + 1) > g_log->max_size )
+                    log_reset_flag = true;
+                else{
+                    /*  Add the new compressed chunk to the log buffer,
+                        over-writing the last part of the log that was just
+                        compressed */
+                    memcpy(&g_log->buf[zip_pos], out, zip_size);
+                    g_log->zip_size[g_log->zip_count] = zip_size;
+                    g_log->zip_count++;
+                    g_log->curr_pos = zip_pos + zip_size;
+
+                    /*  Set a NULL ending */
+                    g_log->buf[g_log->curr_pos] ='\0';
+
+                    /*  Only if there is space to add another compressed chunk,
+                        prepare its start position. */
+                    if( g_log->zip_count < ZIP_COUNT_MAX )
+                        g_log->zip_pos[g_log->zip_count] = g_log->curr_pos;
+                }
+            }
+        }
+
+        /* There was some space-shortage problem. Reset the log. */
+        if ( log_reset_flag ){
+            g_log->curr_pos = 0;
+            for( uint8_t i = 0; i < ZIP_COUNT_MAX; i++ ) g_log->zip_pos[i] = 0;
+            for( uint8_t i = 0; i < ZIP_COUNT_MAX; i++ ) g_log->zip_size[i] = 0;
+            g_log->zip_count = 0;
+        }
     }
+
     memcpy(&g_log->buf[g_log->curr_pos], str, count);
     g_log->curr_pos += count; 
 
