@@ -46,9 +46,12 @@
 #include <misc.h>
 #include <hash.h>
 #include <tpm.h>
+#include <loader.h>
+#include <tb_error.h>
 #include <txt/mtrrs.h>
 #include <txt/config_regs.h>
 #include <txt/heap.h>
+#include <txt/txt.h>
 #endif
 
 /*
@@ -174,7 +177,7 @@ static void print_evt_log(const event_log_container_t *elog)
     printk(TBOOT_DETA"\t\t\t   PCREventVer: %u.%u\n",
            elog->pcr_event_ver_major, elog->pcr_event_ver_minor);
     printk(TBOOT_DETA"\t\t\t          Size: %u\n", elog->size);
-    printk(TBOOT_DETA"\t\t\t  EventsOffset: [%u,%u)\n",
+    printk(TBOOT_DETA"\t\t\t  EventsOffset: [%u,%u]\n",
            elog->pcr_events_offset, elog->next_event_offset);
 
     const tpm12_pcr_event_t *curr, *next;
@@ -257,13 +260,15 @@ void print_event_2(void *evt, uint16_t alg)
 
     printk(TBOOT_DETA"\t\t\t Event:\n");
     printk(TBOOT_DETA"\t\t\t     PCRIndex: %u\n", *((uint32_t *)next));
-    if ( *((uint32_t *)next) > 24 ) {
-        printk(TBOOT_DETA"\t\t\t           Wrong Event Log.\n");
-        return;
-    }
-
+    
+    if ( *((uint32_t *)next) > 24 && *((uint32_t *)next) != 0xFF ) {
+         printk(TBOOT_DETA"\t\t\t           Wrong Event Log.\n");
+         return;
+     }
+    
     next += sizeof(uint32_t);
     printk(TBOOT_DETA"\t\t\t         Type: 0x%x\n", *((uint32_t *)next));
+
     if ( *((uint32_t *)next) > 0xFFF ) {
         printk(TBOOT_DETA"\t\t\t           Wrong Event Log.\n");
         return;
@@ -287,6 +292,102 @@ void print_event_2(void *evt, uint16_t alg)
          printk(TBOOT_DETA"\n");
 }
 
+uint32_t print_event_2_1_log_header(void *evt)
+{
+    tcg_pcr_event *evt_ptr = (tcg_pcr_event *)evt;
+    tcg_efi_specid_event_strcut *evt_data_ptr = (tcg_efi_specid_event_strcut *) evt_ptr->event_data;
+
+    printk(TBOOT_DETA"\t TCG Event Log Header:\n");
+    printk(TBOOT_DETA"\t\t       pcr_index: %u\n", evt_ptr->pcr_index);
+    printk(TBOOT_DETA"\t\t      event_type: %u\n", evt_ptr->event_type);
+    printk(TBOOT_DETA"\t\t          digest: %s\n", evt_ptr->digest);
+    printk(TBOOT_DETA"\t\t event_data_size: %u\n", evt_ptr->event_data_size);
+
+    // print out event log header data
+
+    printk(TBOOT_DETA"\t\t 	   header event data:  \n"); 
+    printk(TBOOT_DETA"\t\t\t              signature: %s\n", evt_data_ptr->signature);
+    printk(TBOOT_DETA"\t\t\t         platform_class: %u\n", evt_data_ptr->platform_class);
+    printk(TBOOT_DETA"\t\t\t     spec_version_major: %u\n", evt_data_ptr->spec_version_major);
+    printk(TBOOT_DETA"\t\t\t     spec_version_minor: %u\n", evt_data_ptr->spec_version_minor);
+    printk(TBOOT_DETA"\t\t\t            spec_errata: %u\n", evt_data_ptr->spec_errata);
+    printk(TBOOT_DETA"\t\t\t             uintn_size: %u\n", evt_data_ptr->uintn_size);
+    printk(TBOOT_DETA"\t\t\t   number_of_algorithms: %u\n", evt_data_ptr->number_of_algorithms);
+
+    for ( uint32_t i = 0; i < evt_data_ptr->number_of_algorithms; i++ ) {
+        printk(TBOOT_DETA"\t\t\t\t   algorithm_id: 0x%x \n", evt_data_ptr->digestSizes[i].algorithm_id);
+        printk(TBOOT_DETA"\t\t\t\t    digest_size: %u\n", evt_data_ptr->digestSizes[i].digest_size);
+    }
+
+    printk(TBOOT_DETA"\t\t\t       vendor_info: %u bytes\n", evt_data_ptr->vendor_info_size);
+    print_hex(NULL, evt_data_ptr->vendor_info, evt_data_ptr->vendor_info_size);
+
+    return evt_ptr->event_data_size;
+}
+
+uint32_t print_event_2_1(void *evt)
+{
+    tcg_pcr_event2 *evt_ptr = (tcg_pcr_event2 *)evt;
+    uint8_t *evt_data_ptr;
+    uint16_t hash_alg;
+    uint32_t event_size = 0;
+    printk(TBOOT_DETA"\t\t\t TCG Event:\n");
+    printk(TBOOT_DETA"\t\t\t      pcr_index: %u\n", evt_ptr->pcr_index);
+    printk(TBOOT_DETA"\t\t\t     event_type: 0x%x\n", evt_ptr->event_type);
+    printk(TBOOT_DETA"\t\t\t          count: %u\n", evt_ptr->digest.count);
+    if (evt_ptr->digest.count != 0) {
+        evt_data_ptr = (uint8_t *)evt_ptr->digest.digests[0].digest;
+        hash_alg = evt_ptr->digest.digests[0].hash_alg;
+        for ( uint32_t i = 0; i < evt_ptr->digest.count; i++ ) { 
+            switch (hash_alg) {
+                case TB_HALG_SHA1:
+                    printk(TBOOT_INFO"SHA1: \n");
+                    print_hex(NULL, evt_data_ptr, SHA1_LENGTH);
+                    evt_data_ptr += SHA1_LENGTH;
+                    break;
+ 
+                case TB_HALG_SHA256:
+                    printk(TBOOT_INFO"SHA256: \n");
+                    print_hex(NULL, evt_data_ptr, SHA256_LENGTH);
+                    evt_data_ptr += SHA256_LENGTH;
+                    break;
+ 
+                case TB_HALG_SM3:
+                    printk(TBOOT_INFO"SM3_256: \n");
+                    print_hex(NULL, evt_data_ptr, SM3_LENGTH);
+                    evt_data_ptr += SM3_LENGTH;
+                    break;
+ 
+                case TB_HALG_SHA384:
+                    printk(TBOOT_INFO"SHA384: \n");
+                    print_hex(NULL, evt_data_ptr, SHA384_LENGTH);
+                    evt_data_ptr += SHA384_LENGTH;				
+                    break;
+ 
+                case TB_HALG_SHA512:
+                    printk(TBOOT_INFO"SHA512:  \n");
+                    print_hex(NULL, evt_data_ptr, SHA512_LENGTH);
+                    evt_data_ptr += SHA512_LENGTH;
+                    break;
+                default:
+                    printk(TBOOT_ERR"Unsupported algorithm: %u\n", evt_ptr->digest.digests[i].hash_alg);
+            }
+            hash_alg = (uint16_t)*evt_data_ptr;
+            evt_data_ptr += sizeof(uint16_t);
+        }
+        evt_data_ptr -= sizeof(uint16_t);
+        event_size = (uint32_t)*evt_data_ptr;
+        printk(TBOOT_DETA"\t\t\t     event_data: %u bytes", event_size);
+        evt_data_ptr += sizeof(uint32_t);
+        print_hex("\t\t\t     ", evt_data_ptr, event_size);
+    }
+    else { 
+        printk(TBOOT_DETA"sth wrong in TCG event log: algoritm count = %u\n", evt_ptr->digest.count);
+        evt_data_ptr= (uint8_t *)evt +12;
+    }
+    return (evt_data_ptr + event_size - (uint8_t *)evt);
+}
+
 static void print_evt_log_ptr_elt_2(const heap_ext_data_element_t *elt)
 {
     const heap_event_log_ptr_elt2_t *elog_elt =
@@ -302,7 +403,7 @@ static void print_evt_log_ptr_elt_2(const heap_ext_data_element_t *elt)
         printk(TBOOT_DETA"\t\t\t Log Descrption:\n");
         printk(TBOOT_DETA"\t\t\t             Alg: %u\n", log_descr->alg);
         printk(TBOOT_DETA"\t\t\t            Size: %u\n", log_descr->size);
-        printk(TBOOT_DETA"\t\t\t    EventsOffset: [%u,%u)\n",
+        printk(TBOOT_DETA"\t\t\t    EventsOffset: [%u,%u]\n",
                 log_descr->pcr_events_offset,
                 log_descr->next_event_offset);
 
@@ -317,10 +418,26 @@ static void print_evt_log_ptr_elt_2(const heap_ext_data_element_t *elt)
             return;
 
         void *curr, *next;
-        curr = (void *)(unsigned long)log_descr->phys_addr +
+
+        curr = (void *)(unsigned long)log_descr->phys_addr + 
                 log_descr->pcr_events_offset;
         next = (void *)(unsigned long)log_descr->phys_addr +
                 log_descr->next_event_offset;
+        
+        //It is required for each of the non-SHA1 event log the first entry to be the following
+        //TPM1.2 style TCG_PCR_EVENT record specifying type of the log:
+        //TCG_PCR_EVENT.PCRIndex = 0
+        //TCG_PCR_EVENT.EventType = 0x03 // EV_NO_ACTION per TCG EFI
+                                       // Platform specification
+        //TCG_PCR_EVENT.Digest = {00…00} // 20 zeros
+        //TCG_PCR_EVENT.EventDataSize = sizeof(TCG_LOG_DESCRIPTOR).
+        //TCG_PCR_EVENT.EventData = TCG_LOG_DESCRIPTOR
+        //The digest of this record MUST NOT be extended into any PCR.
+
+        if (log_descr->alg != TB_HALG_SHA1){
+            print_event_2(curr, TB_HALG_SHA1);
+            curr += sizeof(tpm12_pcr_event_t) + sizeof(tpm20_log_descr_t);
+        }
 
         while ( curr < next ) {
             print_event_2(curr, log_descr->alg);
@@ -330,6 +447,35 @@ static void print_evt_log_ptr_elt_2(const heap_ext_data_element_t *elt)
     }
 }
 
+static void print_evt_log_ptr_elt_2_1(const heap_ext_data_element_t *elt)
+{
+    const heap_event_log_ptr_elt2_1_t *elog_elt = (const heap_event_log_ptr_elt2_1_t *)elt->data;
+   
+    printk(TBOOT_DETA"\t TCG EVENT_LOG_PTR:\n");
+    printk(TBOOT_DETA"\t\t       type: %d\n", elt->type);
+    printk(TBOOT_DETA"\t\t       size: %u\n", elt->size);
+    printk(TBOOT_DETA"\t TCG Event Log Descrption:\n");
+    printk(TBOOT_DETA"\t     allcoated_event_container_size: %u\n", elog_elt->allcoated_event_container_size);
+    printk(TBOOT_DETA"\t                       EventsOffset: [%u,%u]\n", 
+           elog_elt->first_record_offset, elog_elt->next_record_offset);
+
+    if (elog_elt->first_record_offset == elog_elt->next_record_offset) {
+        printk(TBOOT_DETA"\t\t\t No Event Log found.\n");
+        return;
+    }
+    void *curr, *next;
+
+    curr = (void *)(unsigned long)elog_elt->phys_addr + elog_elt->first_record_offset;
+    next = (void *)(unsigned long)elog_elt->phys_addr + elog_elt->next_record_offset;               
+    uint32_t event_header_data_size = print_event_2_1_log_header(curr);
+		
+    curr += sizeof(tcg_pcr_event) + event_header_data_size;
+    while ( curr < next ) {
+        curr += print_event_2_1(curr);
+    }
+}
+
+
 static bool verify_evt_log_ptr_elt_2(const heap_ext_data_element_t *elt)
 {
     if ( !elt )
@@ -360,6 +506,9 @@ static void print_ext_data_elts(const heap_ext_data_element_t elts[])
             case HEAP_EXTDATA_TYPE_TPM_EVENT_LOG_PTR_2:
                 print_evt_log_ptr_elt_2(elt);
                 break;
+            case HEAP_EXTDATA_TYPE_TPM_EVENT_LOG_PTR_2_1:
+                print_evt_log_ptr_elt_2_1(elt);
+                break;
             default:
                 printk(TBOOT_WARN"\t\t unknown element:  type: %u, size: %u\n",
                        elt->type, elt->size);
@@ -470,8 +619,8 @@ bool verify_bios_data(const txt_heap_t *txt_heap)
     if ( bios_data->version > 4 )
         printk(TBOOT_WARN"unsupported BIOS data version (%u)\n", bios_data->version);
 
-    /* all TXT-capable CPUs support at least 2 cores */
-    if ( bios_data->num_logical_procs < 2 ) {
+    /* all TXT-capable CPUs support at least 1 core */
+    if ( bios_data->num_logical_procs < 1 ) {
         printk(TBOOT_ERR"BIOS data has incorrect num_logical_procs (%u)\n",
                bios_data->num_logical_procs);
         return false;
@@ -562,19 +711,25 @@ uint64_t calc_os_sinit_data_size(uint32_t version)
             2 * sizeof(heap_ext_data_element_t) +
             sizeof(heap_event_log_ptr_elt_t)
     };
+    struct tpm_if *tpm = get_tpm();
+    int log_type = get_evtlog_type();
 
-    if ( g_tpm->major == TPM20_VER_MAJOR ) {
-        u32 count;
-        if ( g_tpm->extpol == TB_EXTPOL_AGILE )
-            count = g_tpm->banks;
-        else if ( g_tpm->extpol == TB_EXTPOL_EMBEDDED )
-            count = g_tpm->alg_count;
-        else
-            count = 1;
-
+    if ( log_type == EVTLOG_TPM2_TCG ) {
         size[2] = sizeof(os_sinit_data_t) + sizeof(uint64_t) +
-            2 * sizeof(heap_ext_data_element_t) +
-            4 + count*sizeof(heap_event_log_descr_t);
+        2 * sizeof(heap_ext_data_element_t) +
+        sizeof(heap_event_log_ptr_elt2_1_t);
+    } else if (log_type == EVTLOG_TPM2_LEGACY) {
+        u32 count;
+        if ( tpm->extpol == TB_EXTPOL_AGILE )
+            count = tpm->banks;
+        else
+            if ( tpm->extpol == TB_EXTPOL_EMBEDDED )
+                count = tpm->alg_count;
+            else
+                count = 1;
+        size[2] = sizeof(os_sinit_data_t) + sizeof(uint64_t) +
+            2 * sizeof(heap_ext_data_element_t) + 4 +
+            count*sizeof(heap_event_log_descr_t);
     }
 
     if ( version >= 6 )
@@ -583,6 +738,14 @@ uint64_t calc_os_sinit_data_size(uint32_t version)
         return size[version - MIN_OS_SINIT_DATA_VER];
 }
 
+void print_os_sinit_data_vtdpmr(const os_sinit_data_t *os_sinit_data)
+{
+    printk(TBOOT_DETA"\t vtd_pmr_lo_base: 0x%Lx\n", os_sinit_data->vtd_pmr_lo_base);
+    printk(TBOOT_DETA"\t vtd_pmr_lo_size: 0x%Lx\n", os_sinit_data->vtd_pmr_lo_size);
+    printk(TBOOT_DETA"\t vtd_pmr_hi_base: 0x%Lx\n", os_sinit_data->vtd_pmr_hi_base);
+    printk(TBOOT_DETA"\t vtd_pmr_hi_size: 0x%Lx\n", os_sinit_data->vtd_pmr_hi_size);
+}
+
 void print_os_sinit_data(const os_sinit_data_t *os_sinit_data)
 {
     printk(TBOOT_DETA"os_sinit_data (@%p, %Lx):\n", os_sinit_data,
@@ -593,13 +756,9 @@ void print_os_sinit_data(const os_sinit_data_t *os_sinit_data)
     printk(TBOOT_DETA"\t mle_size: 0x%Lx (%Lu)\n", os_sinit_data->mle_size,
            os_sinit_data->mle_size);
     printk(TBOOT_DETA"\t mle_hdr_base: 0x%Lx\n", os_sinit_data->mle_hdr_base);
-    printk(TBOOT_DETA"\t vtd_pmr_lo_base: 0x%Lx\n", os_sinit_data->vtd_pmr_lo_base);
-    printk(TBOOT_DETA"\t vtd_pmr_lo_size: 0x%Lx\n", os_sinit_data->vtd_pmr_lo_size);
-    printk(TBOOT_DETA"\t vtd_pmr_hi_base: 0x%Lx\n", os_sinit_data->vtd_pmr_hi_base);
-    printk(TBOOT_DETA"\t vtd_pmr_hi_size: 0x%Lx\n", os_sinit_data->vtd_pmr_hi_size);
+    print_os_sinit_data_vtdpmr(os_sinit_data);
     printk(TBOOT_DETA"\t lcp_po_base: 0x%Lx\n", os_sinit_data->lcp_po_base);
-    printk(TBOOT_DETA"\t lcp_po_size: 0x%Lx (%Lu)\n", os_sinit_data->lcp_po_size,
-           os_sinit_data->lcp_po_size);
+    printk(TBOOT_DETA"\t lcp_po_size: 0x%Lx (%Lu)\n", os_sinit_data->lcp_po_size, os_sinit_data->lcp_po_size);
     print_txt_caps("\t ", os_sinit_data->capabilities);
     if ( os_sinit_data->version >= 5 )
         printk(TBOOT_DETA"\t efi_rsdt_ptr: 0x%Lx\n", os_sinit_data->efi_rsdt_ptr);
