@@ -73,9 +73,11 @@ static unsigned int g_cpuid_ext_feat_info;
 static unsigned long g_feat_ctrl_msr;
 
 
-static void read_processor_info(void)
+static bool read_processor_info(void)
 {
     unsigned long f1, f2;
+     /* eax: regs[0], ebx: regs[1], ecx: regs[2], edx: regs[3] */
+    uint32_t regs[4];
 
     /* is CPUID supported? */
     /* (it's supported if ID flag in EFLAGS can be set and cleared) */
@@ -93,13 +95,28 @@ static void read_processor_info(void)
         : "ir" (X86_EFLAGS_ID));
     if ( ((f1^f2) & X86_EFLAGS_ID) == 0 ) {
         g_cpuid_ext_feat_info = 0;
-        return;
+        printk(TBOOT_ERR"CPUID instruction is not supported.\n");
+        return false;
     }
 
+    do_cpuid(0, regs);
+    if ( regs[1] != 0x756e6547        /* "Genu" */
+         || regs[2] != 0x6c65746e     /* "ntel" */
+         || regs[3] != 0x49656e69 ) { /* "ineI" */
+        g_cpuid_ext_feat_info = 0;
+        printk(TBOOT_ERR"Non-Intel CPU detected.\n");
+        return false;
+    }
     g_cpuid_ext_feat_info = cpuid_ecx(1);
 
-    g_feat_ctrl_msr = rdmsr(MSR_IA32_FEATURE_CONTROL);
-    printk(TBOOT_DETA"IA32_FEATURE_CONTROL_MSR: %08lx\n", g_feat_ctrl_msr);
+    /* read feature control msr only if processor supports VMX or SMX instructions */
+    if ( (g_cpuid_ext_feat_info & CPUID_X86_FEATURE_VMX) ||
+         (g_cpuid_ext_feat_info & CPUID_X86_FEATURE_SMX) ) {
+        g_feat_ctrl_msr = rdmsr(MSR_IA32_FEATURE_CONTROL);
+        printk(TBOOT_DETA"IA32_FEATURE_CONTROL_MSR: %08lx\n", g_feat_ctrl_msr);        
+    }
+
+    return true;
 }
 
 static bool supports_vmx(void)
@@ -166,15 +183,16 @@ static bool supports_smx(void)
 
 bool use_mwait(void)
 {
-    return get_tboot_mwait() && 
-           (g_cpuid_ext_feat_info & CPUID_X86_FEATURE_XMM3);
+    return get_tboot_mwait() && (g_cpuid_ext_feat_info & CPUID_X86_FEATURE_XMM3);
 }
 
 tb_error_t supports_txt(void)
 {
     capabilities_t cap;
 
-    read_processor_info();
+    /* processor must support cpuid and must be Intel CPU */
+    if ( !read_processor_info() )
+        return TB_ERR_SMX_NOT_SUPPORTED;
 
     /* processor must support SMX */
     if ( !supports_smx() )
@@ -300,8 +318,8 @@ static bool verify_vtd_pmrs(txt_heap_t *txt_heap)
          (tmp_os_sinit_data.vtd_pmr_hi_size !=
           os_sinit_data->vtd_pmr_hi_size) ) {
         printk(TBOOT_ERR"OS to SINIT data VT-d PMR settings do not match:\n");
-        print_os_sinit_data(&tmp_os_sinit_data);
-        print_os_sinit_data(os_sinit_data);
+        print_os_sinit_data_vtdpmr(&tmp_os_sinit_data);
+        print_os_sinit_data_vtdpmr(os_sinit_data);
         return false;
     }
 
@@ -353,6 +371,10 @@ tb_error_t txt_verify_platform(void)
     if ( err != TB_ERR_NONE )
         return err;
 
+    if ( !vtd_bios_enabled() ) {
+        return TB_ERR_VTD_NOT_SUPPORTED;
+    }
+
     /* check is TXT_RESET.STS is set, since if it is SENTER will fail */
     txt_ests_t ests = (txt_ests_t)read_pub_config_reg(TXTCR_ESTS);
     if ( ests.txt_reset_sts ) {
