@@ -62,7 +62,6 @@ typedef uint8_t multiboot_info_t;
 void print_hex(const char* prefix, const void *start, size_t len);
 #include "../include/hash.h"
 #include "../tboot/include/txt/heap.h"
-
 #include "../tboot/txt/heap.c"
 #include "../tboot/include/lz.h"
 #include "../tboot/common/lz.c"
@@ -188,7 +187,7 @@ static void display_config_regs(void *txt_config_base)
     dpr._raw = read_txt_config_reg(txt_config_base, TXTCR_DPR);
     printf("\tDPR: 0x%016jx\n", dpr._raw);
     printf("\t    lock: %s\n", bit_to_str(dpr.lock));
-    printf("\t    top: 0x%08x\n", dpr.top << 20);
+    printf("\t    top: 0x%08x\n", (uint32_t)dpr.top << 20);
     printf("\t    size: %uMB (%uB)\n", dpr.size, dpr.size*1024*1024);
 
     /* PUBLIC.KEY */
@@ -243,12 +242,20 @@ static void display_tboot_log(void *log_base)
     /* to uncompress tboot log */ 
     if (log->zip_count > 0) {
         for ( i = 0; i< log->zip_count; i++) {
-            LZ_Uncompress(&log_buf[log->zip_pos[i]], out, log->zip_size[i]);
+            int length = LZ_Uncompress(&log_buf[log->zip_pos[i]], out, log->zip_size[i], sizeof(pbuf));
+            if ( length < 0 )
+               continue;
             /* log is too big for single printk(), so break it up */
             /* print out the uncompressed log */
-            for ( unsigned int curr_pos = 0; curr_pos < 32*1024; curr_pos += sizeof(buf)-1 ) {
-                strncpy(buf, out + curr_pos, sizeof(buf)-1);
-                buf[sizeof(buf)-1] = '\0';
+            for ( int curr_pos = 0; curr_pos < length; curr_pos += sizeof(buf)-1 ) {
+                if ( length - curr_pos >= (int)sizeof(buf) - 1 ) {
+                    strncpy(buf, out + curr_pos, sizeof(buf)-1);
+                    buf[sizeof(buf)-1] = '\0';
+                }
+                else {
+                    strncpy(buf, out + curr_pos, length - curr_pos);
+                    buf[length - curr_pos] = '\0';
+                }
                 printf("%s", buf);
             }
         }
